@@ -332,12 +332,14 @@ struct _glwd_metric {
 struct config_elements {
   char *                                         config_file;
   unsigned int                                   port;
+  size_t                                         max_post_size;
   char *                                         bind_address;
   char *                                         bind_address_metrics;
   char *                                         external_url;
   char *                                         api_prefix;
   char *                                         cookie_domain;
   unsigned int                                   cookie_secure;
+  unsigned int                                   cookie_same_site;
   unsigned int                                   add_x_frame_option_header_deny;
   unsigned short                                 log_mode_args;
   unsigned short                                 log_level_args;
@@ -348,6 +350,9 @@ struct config_elements {
   char *                                         admin_scope;
   char *                                         profile_scope;
   char *                                         allow_origin;
+  char *                                         allow_methods;
+  char *                                         allow_headers;
+  char *                                         expose_headers;
   unsigned int                                   use_secure_connection;
   char *                                         secure_connection_key_file;
   char *                                         secure_connection_pem_file;
