@@ -28,7 +28,7 @@
 #ifndef __GLEWLWYD_H_
 #define __GLEWLWYD_H_
 
-#define _GLEWLWYD_VERSION_ "2.7.0"
+#define _GLEWLWYD_VERSION_ "2.7.1"
 
 #include <jansson.h>
 
@@ -62,12 +62,16 @@
 #define GLEWLWYD_DEFAULT_METRICS_PORT                      4594
 #define GLEWLWYD_DEFAULT_API_PREFIX                        "api"
 #define GLEWLWYD_DEFAULT_ALLOW_ORIGIN                      "*"
+#define GLEWLWYD_DEFAULT_ALLOW_METHODS                     "GET, POST, PUT, DELETE, OPTIONS"
+#define GLEWLWYD_DEFAULT_ALLOW_HEADERS                     "Origin, X-Requested-With, Content-Type, Accept, Bearer, Authorization, DPoP"
+#define GLEWLWYD_DEFAULT_EXPOSE_HEADERS                    "Content-Encoding, Authorization"
 #define GLEWLWYD_DEFAULT_ADMIN_SCOPE                       "g_admin"
 #define GLEWLWYD_DEFAULT_PROFILE_SCOPE                     "g_profile"
 #define GLEWLWYD_DEFAULT_HASH_ALGORITHM                    digest_SHA256
 #define GLEWLWYD_DEFAULT_LOGIN_URL                         "login.html"
 #define GLEWLWYD_DEFAULT_SESSION_KEY                       "GLEWLWYD2_SESSION_ID"
 #define GLEWLWYD_DEFAULT_SESSION_EXPIRATION_COOKIE         5256000 // 10 years
+#define GLEWLWYD_DEFAULT_MAX_POST_SIZE                     (16*1024*1024)+1024
 
 #define GLEWLWYD_DEFAULT_SESSION_EXPIRATION_PASSWORD       40320   // 4 weeks
 #define GLEWLWYD_RESET_PASSWORD_DEFAULT_SESSION_EXPIRATION 2592000 // 30 days
@@ -103,6 +107,7 @@
 
 // Environment variables names
 #define GLEWLWYD_ENV_PORT                        "GLWD_PORT"
+#define GLEWLWYD_ENV_MAX_POST_SIZE               "GLWD_MAX_POST_SIZE"
 #define GLEWLWYD_ENV_BIND_ADDRESS                "GLWD_BIND_ADDRESS"
 #define GLEWLWYD_ENV_API_PREFIX                  "GLWD_API_PREFIX"
 #define GLEWLWYD_ENV_EXTERNAL_URL                "GLWD_EXTERNAL_URL"
@@ -111,11 +116,15 @@
 #define GLEWLWYD_ENV_STATIC_FILES_PATH           "GLWD_STATIC_FILES_PATH"
 #define GLEWLWYD_ENV_STATIC_FILES_MIME_TYPES     "GLWD_STATIC_FILES_MIME_TYPES"
 #define GLEWLWYD_ENV_ALLOW_ORIGIN                "GLWD_ALLOW_ORIGIN"
+#define GLEWLWYD_ENV_ALLOW_METHODS               "GLWD_ALLOW_METHODS"
+#define GLEWLWYD_ENV_ALLOW_HEADERS               "GLWD_ALLOW_HEADERS"
+#define GLEWLWYD_ENV_EXPOSE_HEADERS              "GLWD_EXPOSE_HEADERS"
 #define GLEWLWYD_ENV_LOG_MODE                    "GLWD_LOG_MODE"
 #define GLEWLWYD_ENV_LOG_LEVEL                   "GLWD_LOG_LEVEL"
 #define GLEWLWYD_ENV_LOG_FILE                    "GLWD_LOG_FILE"
 #define GLEWLWYD_ENV_COOKIE_DOMAIN               "GLWD_COOKIE_DOMAIN"
 #define GLEWLWYD_ENV_COOKIE_SECURE               "GLWD_COOKIE_SECURE"
+#define GLEWLWYD_ENV_COOKIE_SAME_SITE            "GLWD_COOKIE_SAME_SITE"
 #define GLEWLWYD_ENV_ADD_X_FRAME_DENY            "GLWD_ADD_X_FRAME_DENY"
 #define GLEWLWYD_ENV_SESSION_EXPIRATION          "GLWD_SESSION_EXPIRATION"
 #define GLEWLWYD_ENV_SESSION_KEY                 "GLWD_SESSION_KEY"
@@ -144,6 +153,20 @@
 #define GLEWLWYD_ENV_METRICS_ADMIN               "GLWD_METRICS_ADMIN"
 #define GLEWLWYD_ENV_METRICS_BIND_ADDRESS        "GLWD_METRICS_BIND_ADDRESS"
 
+struct send_mail_content_struct {
+  char                   * host;
+  int                      port;
+  int                      use_tls;
+  int                      verify_certificate;
+  char                   * user;
+  char                   * password;
+  char                   * from;
+  char                   * content_type;
+  char                   * email;
+  char                   * subject;
+  char                   * body;
+};
+
 // Main functions and misc functions
 int build_config_from_env(struct config_elements * config);
 int  build_config_from_file(struct config_elements * config);
@@ -174,6 +197,9 @@ struct _user_auth_scheme_module * get_user_auth_scheme_module_lib(struct config_
 struct _plugin_module_instance * get_plugin_module_instance(struct config_elements * config, const char * name);
 struct _plugin_module * get_plugin_module_lib(struct config_elements * config, const char * name);
 char * get_ip_data(struct config_elements * config, const char * ip_address);
+const char * get_template_property(json_t * j_params, const char * template_property, const char * user_lang, const char * property_field);
+char * complete_template(const char * template, ...);
+void * thread_send_mail(void * args);
 
 // Modules generic functions
 int module_parameters_check(const char * module_parameters);
@@ -203,7 +229,7 @@ int delete_user_session_from_hash(struct config_elements * config, const char *
 json_t * user_set_profile(struct config_elements * config, const char * username, json_t * j_profile);
 int user_delete_profile(struct config_elements * config, const char * username);
 json_t * user_get_profile(struct config_elements * config, const char * username);
-int user_update_password(struct config_elements * config, const char * username, const char * old_password, const char ** new_passwords, size_t new_passwords_len);
+int user_update_password(struct config_elements * config, const char * username, const char * old_password, const char ** new_passwords, size_t new_passwords_len, const char * ip_address);
 int user_set_password(struct config_elements * config, const char * username, const char ** new_passwords, size_t new_passwords_len);
 json_t * get_scheme_list_for_user(struct config_elements * config, const char * username);
 
