@@ -59,7 +59,7 @@ char * get_file_content(const char * file_path) {
     fseek (f, 0, SEEK_END);
     length = ftell (f);
     fseek (f, 0, SEEK_SET);
-    buffer = o_malloc((length+1)*sizeof(char));
+    buffer = o_malloc((length+1));
     if (buffer) {
       res = fread (buffer, 1, length, f);
       if (res != length) {
@@ -109,7 +109,7 @@ char * get_client_hostname(const struct _u_request * request) {
   if (ip_source != NULL) {
     hostname = o_strdup(ip_source);
     if (!getaddrinfo(ip_source, NULL, &hints, &lookup)) {
-      if (o_strlen(lookup->ai_canonname)) {
+      if (!o_strnullempty(lookup->ai_canonname)) {
         hostname = mstrcatf(hostname, " - %s", lookup->ai_canonname);
       }
       freeaddrinfo(lookup);
@@ -208,7 +208,7 @@ char * join_json_string_array(json_t * j_array, const char * separator) {
   
   if (j_array != NULL && json_is_array(j_array)) {
     json_array_foreach(j_array, index, j_element) {
-      if (json_is_string(j_element) && json_string_length(j_element)) {
+      if (json_is_string(j_element) && !json_string_null_or_empty(j_element)) {
         if (str_result == NULL) {
           str_result = o_strdup(json_string_value(j_element));
         } else {
@@ -558,3 +558,7 @@ int check_result_value(json_t * result, const int value) {
   return (json_is_integer(json_object_get(result, "result")) && 
           json_integer_value(json_object_get(result, "result")) == value);
 }
+
+int json_string_null_or_empty(json_t * j_str) {
+  return o_strnullempty(json_string_value(j_str));
+}
