@@ -103,7 +103,6 @@ static int get_certificate_id(gnutls_x509_crt_t cert, unsigned char * cert_id, s
   gnutls_datum_t dat;
   dat.data = NULL;
   
-  
   if (gnutls_x509_crt_export2(cert, GNUTLS_X509_FMT_DER, &dat) >= 0) {
     if (gnutls_fingerprint(GNUTLS_DIG_SHA256, &dat, cert_digest, &cert_digest_len) == GNUTLS_E_SUCCESS) {
       if (o_base64_encode(cert_digest, cert_digest_len, cert_id, cert_id_len)) {
@@ -133,27 +132,19 @@ static json_t * parse_certificate(const char * x509_data, int der_format) {
   size_t key_id_enc_len = 256, dn_len = 0, issuer_dn_len = 0;
   time_t expires_at = 0, issued_at = 0;
   int ret;
-  unsigned char * der_dec = NULL, key_id_enc[257] = {0};
-  size_t der_dec_len = 0;
+  unsigned char key_id_enc[257] = {0};
+  struct _o_datum dat = {0, NULL};
   
   if (!o_strnullempty(x509_data)) {
     if (!gnutls_x509_crt_init(&cert)) {
       if (der_format) {
         cert_dat.data = NULL;
         cert_dat.size = 0;
-        if (o_base64_decode((const unsigned char *)x509_data, o_strlen(x509_data), NULL, &der_dec_len)) {
-          if ((der_dec = o_malloc(der_dec_len+4)) != NULL) {
-            if (o_base64_decode((const unsigned char *)x509_data, o_strlen(x509_data), der_dec, &der_dec_len)) {
-              cert_dat.data = der_dec;
-              cert_dat.size = der_dec_len;
-            } else {
-              y_log_message(Y_LOG_LEVEL_ERROR, "parse_certificate - Error o_base64_decode (2)");
-            }
-          } else {
-            y_log_message(Y_LOG_LEVEL_ERROR, "parse_certificate - Error allocating resources for der_dec");
-          }
+        if (o_base64_decode_alloc((const unsigned char *)x509_data, o_strlen(x509_data), &dat)) {
+          cert_dat.data = dat.data;
+          cert_dat.size = dat.size;
         } else {
-          y_log_message(Y_LOG_LEVEL_ERROR, "parse_certificate - Error o_base64_decode (1)");
+          y_log_message(Y_LOG_LEVEL_ERROR, "parse_certificate - Error o_base64_decode_alloc");
         }
       } else {
         cert_dat.data = (unsigned char *)x509_data;
@@ -224,7 +215,7 @@ static json_t * parse_certificate(const char * x509_data, int der_format) {
       y_log_message(Y_LOG_LEVEL_ERROR, "parse_certificate - Error gnutls_x509_crt_init");
       j_return = json_pack("{si}", "result", G_ERROR);
     }
-    o_free(der_dec);
+    o_free(dat.data);
   } else {
     j_return = json_pack("{si}", "result", G_ERROR_PARAM);
   }
@@ -1247,6 +1238,7 @@ int user_auth_scheme_module_can_use(struct config_module * config, const char *
  * @return value: a json_t * value with the following pattern:
  *                {
  *                  result: number (G_OK on success, another value on error)
+ *                  updated: boolean (true if the scheme has been registered or updated, optional)
  *                  response: JSON object, optional
  *                }
  * 
@@ -1327,7 +1319,7 @@ json_t * user_auth_scheme_module_register(struct config_module * config, const s
   } else if (json_object_get(((struct _cert_param *)cls)->j_parameters, "use-scheme-storage") == json_true()) {
     if (0 == o_strcmp("upload-certificate", json_string_value(json_object_get(j_scheme_data, "register")))) {
       if ((ret = add_user_certificate_scheme_storage(config, ((struct _cert_param *)cls)->j_parameters, json_string_value(json_object_get(j_scheme_data, "x509")), username, u_map_get_case(http_request->map_header, "user-agent"))) == G_OK) {
-        j_return = json_pack("{si}", "result", G_OK);
+        j_return = json_pack("{siso}", "result", G_OK, "updated", json_true());
       } else if (ret == G_ERROR_PARAM) {
         j_return = json_pack("{si}", "result", G_ERROR_PARAM);
       } else {
@@ -1338,7 +1330,7 @@ json_t * user_auth_scheme_module_register(struct config_module * config, const s
       if ((((struct _cert_param *)cls)->cert_source & G_CERT_SOURCE_TLS) && http_request->client_cert != NULL) {
         if ((x509_data = ulfius_export_client_certificate_pem(http_request)) != NULL) {
           if ((ret = add_user_certificate_scheme_storage(config, ((struct _cert_param *)cls)->j_parameters, x509_data, username, u_map_get_case(http_request->map_header, "user-agent"))) == G_OK) {
-            j_return = json_pack("{si}", "result", G_OK);
+            j_return = json_pack("{siso}", "result", G_OK, "updated", json_true());
           } else if (ret == G_ERROR_PARAM) {
             j_return = json_pack("{si}", "result", G_ERROR_PARAM);
           } else {
@@ -1352,7 +1344,7 @@ json_t * user_auth_scheme_module_register(struct config_module * config, const s
         }
       } else if ((((struct _cert_param *)cls)->cert_source & G_CERT_SOURCE_HEADER) && (header_cert = u_map_get(http_request->map_header, json_string_value(json_object_get(((struct _cert_param *)cls)->j_parameters, "header-name")))) != NULL) {
         if ((ret = add_user_certificate_scheme_storage(config, ((struct _cert_param *)cls)->j_parameters, header_cert, username, u_map_get_case(http_request->map_header, "user-agent"))) == G_OK) {
-          j_return = json_pack("{si}", "result", G_OK);
+          j_return = json_pack("{siso}", "result", G_OK, "updated", json_true());
         } else if (ret == G_ERROR_PARAM) {
           j_return = json_pack("{si}", "result", G_ERROR_PARAM);
         } else {
@@ -1368,7 +1360,7 @@ json_t * user_auth_scheme_module_register(struct config_module * config, const s
         j_result = get_user_certificate_from_id_scheme_storage(config, ((struct _cert_param *)cls)->j_parameters, username, json_string_value(json_object_get(j_scheme_data, "certificate_id")));
         if (check_result_value(j_result, G_OK)) {
           if (update_user_certificate_enabled_scheme_storage(config, ((struct _cert_param *)cls)->j_parameters, username, json_string_value(json_object_get(j_scheme_data, "certificate_id")), json_object_get(j_scheme_data, "enabled") == json_true()) == G_OK) {
-            j_return = json_pack("{si}", "result", G_OK);
+            j_return = json_pack("{siso}", "result", G_OK, "updated", json_true());
           } else {
             y_log_message(Y_LOG_LEVEL_ERROR, "user_auth_scheme_module_register certificate - Error update_user_certificate_enabled_scheme_storage");
             j_return = json_pack("{si}", "result", G_ERROR);
@@ -1388,7 +1380,7 @@ json_t * user_auth_scheme_module_register(struct config_module * config, const s
         j_result = get_user_certificate_from_id_scheme_storage(config, ((struct _cert_param *)cls)->j_parameters, username, json_string_value(json_object_get(j_scheme_data, "certificate_id")));
         if (check_result_value(j_result, G_OK)) {
           if (delete_user_certificate_scheme_storage(config, ((struct _cert_param *)cls)->j_parameters, username, json_string_value(json_object_get(j_scheme_data, "certificate_id"))) == G_OK) {
-            j_return = json_pack("{si}", "result", G_OK);
+            j_return = json_pack("{siso}", "result", G_OK, "updated", json_true());
           } else {
             y_log_message(Y_LOG_LEVEL_ERROR, "user_auth_scheme_module_register certificate - Error delete_user_certificate_scheme_storage");
             j_return = json_pack("{si}", "result", G_ERROR);
