@@ -5,9 +5,9 @@
  * Authentiation server
  * Users are authenticated via various backend available: database, ldap
  * Using various authentication methods available: password, OTP, send code, etc.
- * 
+ *
  * HOTP/TOTP authentication scheme module
- * 
+ *
  * Copyright 2019-2020 Nicolas Mora <mail@babelouest.org>
  *
  * This program is free software; you can redistribute it and/or
@@ -44,7 +44,7 @@ static int is_current_otp_available(struct config_module * config, json_t * j_pa
   json_t * j_query, * j_result;
   int res, ret;
   char * username_escaped, * username_clause, * last_used_clause;
-  
+
   time(&now);
   username_escaped = h_escape_string_with_quotes(config->conn, username);
   username_clause = msprintf(" = UPPER(%s)", username_escaped);
@@ -94,7 +94,7 @@ static int is_current_otp_available(struct config_module * config, json_t * j_pa
 
 static json_t * is_scheme_parameters_valid(json_t * j_params) {
   json_t * j_return, * j_error;
-  
+
   if (json_is_object(j_params)) {
     j_error = json_array();
     if (j_error != NULL) {
@@ -142,7 +142,7 @@ static json_t * get_otp(struct config_module * config, json_t * j_params, const
   json_t * j_query, * j_result, * j_return;
   int res;
   char * username_escaped, * username_clause;
-  
+
   username_escaped = h_escape_string_with_quotes(config->conn, username);
   username_clause = msprintf(" = UPPER(%s)", username_escaped);
   j_query = json_pack("{sss[sssss]s{s{ssss}sO}}",
@@ -194,7 +194,7 @@ static int update_otp(struct config_module * config, json_t * j_params, const ch
   char * username_escaped, * username_clause, * last_login_clause;
   json_t * j_query;
   int ret;
-  
+
   username_escaped = h_escape_string_with_quotes(config->conn, username);
   username_clause = msprintf(" = UPPER(%s)", username_escaped);
   if (config->conn->type==HOEL_DB_TYPE_MARIADB) {
@@ -240,7 +240,7 @@ static int set_otp(struct config_module * config, json_t * j_params, const char
   json_t * j_query, * j_otp;
   int ret, res, type = (0==o_strcmp(json_string_value(json_object_get(j_scheme_data, "type")), "HOTP")?0:1);
   char * username_escaped, * username_clause;
-  
+
   if (0 != o_strcmp(json_string_value(json_object_get(j_scheme_data, "type")), "NONE")) {
     j_otp = get_otp(config, j_params, username);
     if (check_result_value(j_otp, G_OK)) {
@@ -338,14 +338,14 @@ static int set_otp(struct config_module * config, json_t * j_params, const char
 }
 
 /**
- * 
+ *
  * user_auth_scheme_module_load
- * 
+ *
  * Executed once when Glewlwyd service is started
  * Used to identify the module and to show its parameters on init
  * You can also use it to load resources that are required once for all
  * instance modules for example
- * 
+ *
  * @return value: a json_t * value with the following pattern:
  *                {
  *                  result: number (G_OK on success, another value on error)
@@ -354,7 +354,7 @@ static int set_otp(struct config_module * config, json_t * j_params, const char
  *                  description: string, optional, description for the module
  *                  parameters: object, optional, parameters description for the module
  *                }
- * 
+ *
  *                Example:
  *                {
  *                  result: G_OK,
@@ -368,10 +368,10 @@ static int set_otp(struct config_module * config, json_t * j_params, const char
  *                    }
  *                  }
  *                }
- * 
+ *
  * @parameter config: a struct config_module with acess to some Glewlwyd
  *                    service and data
- * 
+ *
  */
 json_t * user_auth_scheme_module_load(struct config_module * config) {
   UNUSED(config);
@@ -383,18 +383,18 @@ json_t * user_auth_scheme_module_load(struct config_module * config) {
 }
 
 /**
- * 
+ *
  * user_auth_scheme_module_unload
- * 
+ *
  * Executed once when Glewlwyd service is stopped
  * You can also use it to release resources that are required once for all
  * instance modules for example
- * 
+ *
  * @return value: G_OK on success, another value on error
- * 
+ *
  * @parameter config: a struct config_module with acess to some Glewlwyd
  *                    service and data
- * 
+ *
  */
 int user_auth_scheme_module_unload(struct config_module * config) {
   UNUSED(config);
@@ -402,19 +402,19 @@ int user_auth_scheme_module_unload(struct config_module * config) {
 }
 
 /**
- * 
+ *
  * user_auth_scheme_module_init
- * 
+ *
  * Initialize an instance of this module declared in Glewlwyd service.
  * If required, you must dynamically allocate a pointer to the configuration
  * for this instance and pass it to *cls
- * 
+ *
  * @return value: a json_t * value with the following pattern:
  *                {
  *                  result: number (G_OK on success, G_ERROR_PARAM on input parameters error, another value on error)
  *                  error: array of strings containg the list of input errors, mandatory on result G_ERROR_PARAM, ignored otherwise
  *                }
- * 
+ *
  * @parameter config: a struct config_module with acess to some Glewlwyd
  *                    service and data
  * @parameter j_parameters: used to initialize an instance in JSON format
@@ -422,13 +422,13 @@ int user_auth_scheme_module_unload(struct config_module * config) {
  * @parameter mod_name: module name in glewlwyd service
  * @parameter cls: will contain an allocated void * pointer that will be sent back
  *                 as void * in all module functions
- * 
+ *
  */
 json_t * user_auth_scheme_module_init(struct config_module * config, json_t * j_parameters, const char * mod_name, void ** cls) {
   UNUSED(config);
   json_t * j_result = is_scheme_parameters_valid(j_parameters), * j_return;
   char * message;
-  
+
   if (check_result_value(j_result, G_OK)) {
     json_object_set_new(j_parameters, "mod_name", json_string(mod_name));
     *cls = json_incref(j_parameters);
@@ -446,19 +446,19 @@ json_t * user_auth_scheme_module_init(struct config_module * config, json_t * j_
 }
 
 /**
- * 
+ *
  * user_auth_scheme_module_close
- * 
+ *
  * Close an instance of this module declared in Glewlwyd service.
  * You must free the memory previously allocated in
  * the user_auth_scheme_module_init function as void * cls
- * 
+ *
  * @return value: G_OK on success, another value on error
- * 
+ *
  * @parameter config: a struct config_module with acess to some Glewlwyd
  *                    service and data
  * @parameter cls: pointer to the void * cls value allocated in user_auth_scheme_module_init
- * 
+ *
  */
 int user_auth_scheme_module_close(struct config_module * config, void * cls) {
   UNUSED(config);
@@ -467,26 +467,26 @@ int user_auth_scheme_module_close(struct config_module * config, void * cls) {
 }
 
 /**
- * 
+ *
  * user_auth_scheme_module_can_use
- * 
+ *
  * Validate if the user is allowed to use this scheme prior to the
  * authentication or registration
- * 
+ *
  * @return value: GLEWLWYD_IS_REGISTERED - User can use scheme and has registered
  *                GLEWLWYD_IS_AVAILABLE - User can use scheme but hasn't registered
  *                GLEWLWYD_IS_NOT_AVAILABLE - User can't use scheme
- * 
+ *
  * @parameter config: a struct config_module with acess to some Glewlwyd
  *                    service and data
  * @parameter username: username to identify the user
  * @parameter cls: pointer to the void * cls value allocated in user_auth_scheme_module_init
- * 
+ *
  */
 int user_auth_scheme_module_can_use(struct config_module * config, const char * username, void * cls) {
   json_t * j_otp;
   int ret;
-  
+
   j_otp = get_otp(config, (json_t *)cls, username);
   if (check_result_value(j_otp, G_OK)) {
     ret = GLEWLWYD_IS_REGISTERED;
@@ -501,18 +501,19 @@ int user_auth_scheme_module_can_use(struct config_module * config, const char *
 }
 
 /**
- * 
+ *
  * user_auth_scheme_module_register
- * 
+ *
  * Register the scheme for a user
  * Ex: add a certificate, add new TOTP values, etc.
- * 
+ *
  * @return value: a json_t * value with the following pattern:
  *                {
  *                  result: number (G_OK on success, another value on error)
+ *                  updated: boolean (true if the scheme has been registered or updated, optional)
  *                  response: JSON object, optional
  *                }
- * 
+ *
  * @parameter config: a struct config_module with acess to some Glewlwyd
  *                    service and data
  * @parameter http_request: the original struct _u_request from the HTTP API
@@ -520,7 +521,7 @@ int user_auth_scheme_module_can_use(struct config_module * config, const char *
  * @parameter j_scheme_data: additional data used to register the scheme for the user
  *                           in JSON format
  * @parameter cls: pointer to the void * cls value allocated in user_auth_scheme_module_init
- * 
+ *
  */
 json_t * user_auth_scheme_module_register(struct config_module * config, const struct _u_request * http_request, const char * username, json_t * j_scheme_data, void * cls) {
   UNUSED(config);
@@ -528,7 +529,7 @@ json_t * user_auth_scheme_module_register(struct config_module * config, const s
   json_t * j_return = NULL;
   char * secret = NULL, * secret_b32 = NULL;
   size_t secret_len = 0, secret_b32_len = 0;
-  
+
   if (json_is_object(j_scheme_data)) {
     if (json_object_get(j_scheme_data, "generate-secret") == json_true()) {
       secret_len = json_integer_value(json_object_get((json_t *)cls, "secret-minimum-size"))*sizeof(unsigned char);
@@ -587,37 +588,41 @@ json_t * user_auth_scheme_module_register(struct config_module * config, const s
   } else {
     j_return = json_pack("{sis[s]}", "result", G_ERROR_PARAM, "response", "data must be a JSON object");
   }
-  
+
   if (j_return == NULL) {
-    return json_pack("{si}", "result", set_otp(config, (json_t *)cls, username, j_scheme_data)==G_OK?G_OK:G_ERROR);
+    if (set_otp(config, (json_t *)cls, username, j_scheme_data) == G_OK) {
+      return json_pack("{siso}", "result", G_OK, "updated", json_true());
+    } else {
+      return json_pack("{si}", "result", G_ERROR);
+    }
   }
   return j_return;
 }
 
 /**
- * 
+ *
  * user_auth_scheme_module_register_get
- * 
+ *
  * Get the registration value(s) of the scheme for a user
- * 
+ *
  * @return value: a json_t * value with the following pattern:
  *                {
  *                  result: number (G_OK on success, another value on error)
  *                  response: JSON object, optional
  *                }
- * 
+ *
  * @parameter config: a struct config_module with acess to some Glewlwyd
  *                    service and data
  * @parameter http_request: the original struct _u_request from the API, must be casted to be available
  * @parameter username: username to identify the user
  * @parameter cls: pointer to the void * cls value allocated in user_auth_scheme_module_init
- * 
+ *
  */
 json_t * user_auth_scheme_module_register_get(struct config_module * config, const struct _u_request * http_request, const char * username, void * cls) {
   UNUSED(config);
   UNUSED(http_request);
   json_t * j_otp, * j_return;
-  
+
   j_otp = get_otp(config, (json_t *)cls, username);
   if (check_result_value(j_otp, G_OK)) {
     json_object_set(json_object_get(j_otp, "otp"), "digits", json_object_get((json_t *)cls, "otp-length"));
@@ -626,11 +631,11 @@ json_t * user_auth_scheme_module_register_get(struct config_module * config, con
     json_object_set(json_object_get(j_otp, "otp"), "totp-allow", json_object_get((json_t *)cls, "totp-allow")==json_false()?json_false():json_true());
     j_return = json_pack("{sisO}", "result", G_OK, "response", json_object_get(j_otp, "otp"));
   } else if (check_result_value(j_otp, G_ERROR_NOT_FOUND)) {
-    j_return = json_pack("{sis{sssososIsIsI}}", 
-                         "result", G_OK, 
-                         "response", 
-                           "type", "NONE", 
-                           "hotp-allow", json_object_get((json_t *)cls, "hotp-allow")==json_false()?json_false():json_true(), 
+    j_return = json_pack("{sis{sssososIsIsI}}",
+                         "result", G_OK,
+                         "response",
+                           "type", "NONE",
+                           "hotp-allow", json_object_get((json_t *)cls, "hotp-allow")==json_false()?json_false():json_true(),
                            "totp-allow", json_object_get((json_t *)cls, "totp-allow")==json_false()?json_false():json_true(),
                            "hotp-window", json_integer_value(json_object_get((json_t *)cls, "hotp-window")),
                            "totp-window", json_object_get((json_t *)cls, "totp-window")!=NULL?json_integer_value(json_object_get((json_t *)cls, "totp-window")):G_TOTP_DEFAULT_TIME_STEP_SIZE,
@@ -640,47 +645,47 @@ json_t * user_auth_scheme_module_register_get(struct config_module * config, con
     y_log_message(Y_LOG_LEVEL_ERROR, "user_auth_scheme_module_register_get otp - Error get_otp");
   }
   json_decref(j_otp);
-  
+
   return j_return;
 }
 
 /**
- * 
+ *
  * user_auth_scheme_module_deregister
- * 
+ *
  * Deregister the scheme for a user
  * Ex: remove certificates, TOTP values, etc.
- * 
+ *
  * @return value: G_OK on success, even if no data has been removed
  *                G_ERROR on another error
- * 
+ *
  * @parameter config: a struct config_module with acess to some Glewlwyd
  *                    service and data
  * @parameter username: username to identify the user
  * @parameter cls: pointer to the void * cls value allocated in user_auth_scheme_module_init
- * 
+ *
  */
 int user_auth_scheme_module_deregister(struct config_module * config, const char * username, void * cls) {
   json_t * j_scheme_data = json_pack("{ss}", "type", "NONE");
   int ret = set_otp(config, (json_t *)cls, username, j_scheme_data);
   json_decref(j_scheme_data);
-  
+
   return ret;
 }
 
 /**
- * 
+ *
  * user_auth_scheme_module_trigger
- * 
+ *
  * Trigger the scheme for a user
  * Ex: send the code to a device, generate a challenge, etc.
- * 
+ *
  * @return value: a json_t * value with the following pattern:
  *                {
  *                  result: number (G_OK on success, another value on error)
  *                  response: JSON object, optional
  *                }
- * 
+ *
  * @parameter config: a struct config_module with acess to some Glewlwyd
  *                    service and data
  * @parameter http_request: the original struct _u_request from the API, must be casted to be available
@@ -688,7 +693,7 @@ int user_auth_scheme_module_deregister(struct config_module * config, const char
  * @parameter scheme_trigger: data sent to trigger the scheme for the user
  *                           in JSON format
  * @parameter cls: pointer to the void * cls value allocated in user_auth_scheme_module_init
- * 
+ *
  */
 json_t * user_auth_scheme_module_trigger(struct config_module * config, const struct _u_request * http_request, const char * username, json_t * j_scheme_trigger, void * cls) {
   UNUSED(config);
@@ -701,17 +706,17 @@ json_t * user_auth_scheme_module_trigger(struct config_module * config, const st
 }
 
 /**
- * 
+ *
  * user_auth_scheme_module_validate
- * 
+ *
  * Validate the scheme for a user
  * Ex: check the code sent to a device, verify the challenge, etc.
- * 
+ *
  * @return value: G_OK on success
  *                G_ERROR_UNAUTHORIZED if validation fails
  *                G_ERROR_PARAM if error in parameters
  *                G_ERROR on another error
- * 
+ *
  * @parameter config: a struct config_module with acess to some Glewlwyd
  *                    service and data
  * @parameter http_request: the original struct _u_request from the API, must be casted to be available
@@ -719,7 +724,7 @@ json_t * user_auth_scheme_module_trigger(struct config_module * config, const st
  * @parameter j_scheme_data: data sent to validate the scheme for the user
  *                           in JSON format
  * @parameter cls: pointer to the void * cls value allocated in user_auth_scheme_module_init
- * 
+ *
  */
 int user_auth_scheme_module_validate(struct config_module * config, const struct _u_request * http_request, const char * username, json_t * j_scheme_data, void * cls) {
   UNUSED(config);
@@ -728,7 +733,7 @@ int user_auth_scheme_module_validate(struct config_module * config, const struct
   json_t * j_otp;
   char * secret_decoded = NULL;
   size_t secret_decoded_len;
-  
+
   if (json_string_null_or_empty(json_object_get(j_scheme_data, "value")) || json_string_length(json_object_get(j_scheme_data, "value")) != (size_t)json_integer_value(json_object_get((json_t *)cls, "otp-length"))) {
     ret = G_ERROR_UNAUTHORIZED;
   } else if (user_auth_scheme_module_can_use(config, username, cls) == GLEWLWYD_IS_REGISTERED) {
@@ -798,27 +803,27 @@ int user_auth_scheme_module_validate(struct config_module * config, const struct
 }
 
 /**
- * 
+ *
  * user_auth_scheme_module_identify
- * 
+ *
  * Identify the user using the scheme without the username to be previously given
  * This functionality isn't available for all schemes, because the scheme authentification
  * must be triggered without username and the authentication result must contain the username
- * 
+ *
  * @return value: a json_t * value with the following pattern:
  *                {
  *                  result: number (G_OK on success, another value on error)
  *                  username: string value of the user identified - if the function is called within /auth
  *                  response: JSON object, optional - if the function is called within /auth/scheme/trigger
  *                }
- * 
+ *
  * @parameter config: a struct config_module with acess to some Glewlwyd
  *                    service and data
  * @parameter http_request: the original struct _u_request from the API, must be casted to be available
  * @parameter j_scheme_data: data sent to validate the scheme for the user
  *                           in JSON format
  * @parameter cls: pointer to the void * cls value allocated in user_auth_scheme_module_init
- * 
+ *
  */
 json_t * user_auth_scheme_module_identify(struct config_module * config, const struct _u_request * http_request, json_t * j_scheme_data, void * cls) {
   UNUSED(config);
